/* .......................................................................
 Code to control a robot using a Wii Nunchuk
  most of the Wii code was from the Arduino forum ..thanks to the many contributors.
 
 Here is the bitmapping for the standard nunchuk mapping
	    Bit
 Byte     7	 6	 5	 4	 3	 2	 1	 0
   0	 SX<7-----------------------------------------------------0>
   1	 SY<7-----------------------------------------------------0>
   2	 AX<9-----------------------------------------------------2>
   3	 AY<9-----------------------------------------------------2>
   4	 AZ<9-----------------------------------------------------2>
   5	 AZ<1----0>   AY<1-------0>   AX<1-------0>      BC	 BZ
 
 Hardware Arduino with ATMega 328
 Connections 
 SCA to AD4 (Analog4) with pullup of 2k7 to 3.3 volts
 SCL to AD5 (Analog5) with pullup of 2k7 to 3.3 volts
 
 Connections to IR4427P chips are :-
  Pin 3 to 2 (Left Motor)Left2
  Pin 5 to 4 (Left Motor)Left1
  Pin 6 to 2 (Right Motor)Right2
  Pin 9 to 4 (Rigtht Motor)Right1
 
 There isn't anything else as it was designed for a 300g robot, and I wanted some protection


 The Arduino I2C uses 7 bits, therefore the MP address (0xA6) becomes 0x53, and the (0xA4) becomes 0x52
 Modified by Mark Beckett
 jun 2010
 ************************************
 To Do
  Detect if the interface has disconnected and don't go full speed.
  Use something other than a Rugged Circuit driver which needs 8v to run.
  Reduce weight to enable a lifting motor on the front


*/

#include <Wire.h>
#include <string.h>
#include <stdio.h>

//setup the variables, etc
//int ledPin = 13;
int xID;

//Nunchuck
uint8_t outbuf[6];           //nunchuck buffer for data
int cnt = 0;                 //nunchuck buffer counter
int joy_x_axis;              //Joystick x (8 bits)
int joy_y_axis;              //Joystick y (8 bits)
int z_button;
int c_button;

// Motor Movement
int Left2 = 3;  
int Left1 = 5;
int Right2 = 6;  
int Right1 = 9;
int Speed;
int RightAmount;
int LeftAmount;

//Wii Nunchuk
byte data[6];                //buffer for WM+ data

void setup (){
  Serial.begin (19200);

  pinMode(Right1, OUTPUT);
  pinMode(Right2, OUTPUT);
  pinMode(Left1, OUTPUT);
  pinMode(Left2, OUTPUT);
  
  Wire.begin();
  Serial.print ("Finished setup\n");
  Serial.print ("Now detecting WM+\n");
  delay(100);
  
// initiate Wii Nunchuk
    Serial.print ("Initialising Wii Nunchuk ........");
    Wire.beginTransmission(0x52); //0xA4
    Wire.send(0xF0);
    Wire.send(0x55);
    Wire.endTransmission();
    Serial.print (" OK done"); 
    Serial.print ("\r\n");
    delay (100); 
    
    Serial.print ("Set reading address at 0xFA .......");
    Wire.beginTransmission(0x52); //0xA4
    Wire.send(0xFA);
    Wire.endTransmission();
    Serial.print(" OK done");
    Serial.print ("\r\n");
    delay (100);
    Wire.requestFrom (0x52,6);  //0xA4
   outbuf[0] = Wire.receive(); //Serial.print(outbuf[0],HEX);Serial.print(" ");
   outbuf[1] = Wire.receive(); //Serial.print(outbuf[1],HEX);Serial.print(" ");
   outbuf[2] = Wire.receive(); //Serial.print(outbuf[2],HEX);Serial.print(" ");
   outbuf[3] = Wire.receive(); //Serial.print(outbuf[3],HEX);Serial.print(" ");
   outbuf[4] = Wire.receive(); //Serial.print(outbuf[4],HEX);Serial.print(" ");
   outbuf[5] = Wire.receive(); //Serial.print(outbuf[5],HEX);Serial.print(" ");
   //Serial.print ("\r\n");
   xID= outbuf[0] + outbuf[1] + outbuf[2] + outbuf[3] + outbuf[4] + outbuf[5];
   //Serial.print("Extension controller xID = 0x");
   //Serial.print(xID,HEX);
   //delay (200);
   //if (xID == 0xCB) { Serial.print (" Wii Motion plus connected but not activated"); } // 00 00 A6 20 00 05
   //if (xID == 0xCE) { Serial.print (" Wii Motion plus connected and NC activated"); } // 00 00 A4 20 00 05
   //if (xID == 0x00) { Serial.print (" Wii Motion plus not connected"); } // 00 00 00 00 00 00
   //Serial.print ("\r\n");

   delay (500);
   // Now we want to point the read adress to 0xa40008 where the 6 byte data is stored
   // Serial.print ("Set reading address at 0x08 .........");
    Wire.beginTransmission(0x52);
    Wire.send(0x08);
    Wire.endTransmission();
    //Serial.print(" OK done");
    //Serial.print ("\r\n");
    //lcd.clear();
  }
void send_zero (){
    Wire.beginTransmission(0x52);
    Wire.send(0x00);
    Wire.endTransmission();
}
void loop (){
// all sensor data and put them into 6 byte register within the extension controller

  send_zero (); // send the request for next bytes
  delay (100);
  Wire.requestFrom (0x52,6);
  outbuf[0] = Wire.receive();
  outbuf[1] = Wire.receive();
  outbuf[2] = Wire.receive();
  outbuf[3] = Wire.receive();
  outbuf[4] = Wire.receive();
  outbuf[5] = Wire.receive();

  joy_x_axis = outbuf[0];
  joy_y_axis = outbuf[1];
  z_button = (outbuf[5]>>0) & 1;
  c_button = (outbuf[5]>>1) & 1;
  motorcontrol();
  cnt = 0;
 
}

void motorcontrol(){
  // this bit decides which direction and by how much
    
    //Steering
   //Forward and right means the left motor goes faster
   //Backwards and right means the left motor goes faster
   
    //if (joy_x_axis == 128)                            //centre
   // { 
      RightAmount = map(joy_x_axis, 128, 255, 100, 0);
      LeftAmount = map(joy_x_axis, 0, 128, 0, 100);
   // }
    
    if ((joy_y_axis) == 128 && (joy_x_axis) == 128)     //stopped
    {
      //Speed = 0;
      digitalWrite(Right1, LOW);
      digitalWrite(Right2, LOW);
      digitalWrite(Left1, LOW);  
      digitalWrite(Left2, LOW);   
    }
  
    if ((joy_y_axis) > 128)        //forward
    {  
      digitalWrite(Right2, LOW);
      digitalWrite(Left2, LOW);
      Speed = map(joy_y_axis, 128, 255, 0, 255);
      analogWrite(Right1,((Speed * RightAmount)/100));
      analogWrite(Left1,((Speed * LeftAmount)/100));
    }
    
    if ((joy_y_axis) < 128)        //reverse
    {  
      digitalWrite(Right1, LOW);
      digitalWrite(Left1, LOW);
      Speed = map(joy_y_axis, 0, 128, 255, 0);
      analogWrite(Right2,((Speed * RightAmount)/100));
      analogWrite(Left2,((Speed * LeftAmount)/100));

    } 
   
   /*
   
   //Steering
   //Forward and right means the left motor goes faster
   //Backwards and right means the left motor goes faster
   
    if (joy_x_axis == 128)                                    //centre
    { 
      analogWrite(Right2,Speed);
      analogWrite(Left2, Speed);
    }
    
    if ((joy_x_axis) > 128 && (joy_y_axis) != 128)            //right with fwd/rev
    {            
      RightAmount = map(joy_x_axis, 128, 255, 100, 0);
      analogWrite(Right2,((Speed * RightAmount)/100));
      analogWrite(Left2,Speed);
    }
    else if ((joy_x_axis) > 128 && (joy_y_axis) == 128)       //spin to the right with no fwd/rev 
    {  
      LeftAmount = map(joy_x_axis, 128, 255, 0, 255);          // right axis reading fed into left motor
      if (c_button == 1)
      {
        digitalWrite(Right1, LOW);
        digitalWrite(Left1, HIGH);
      }
      else                                                     //reverse
      {
        digitalWrite(Right1, HIGH);
        digitalWrite(Left1, LOW);
      }
      analogWrite(Left2, LeftAmount);                        //Left motor forward
    }
    
    if ((joy_x_axis) < 128 && (joy_y_axis) != 128){            // left with fwd/rev
      LeftAmount = map(joy_x_axis, 128, 0, 100, 0);
      analogWrite(Left2,((Speed * LeftAmount)/100));
      analogWrite(Right2, Speed);
    }
    else if ((joy_x_axis) < 128 && (joy_y_axis) == 128){       // left with no fwd/rev
      RightAmount = map(joy_x_axis, 128, 0, 0, 255);          // left axis reading fed into right motor
      if (c_button == 1)
      {
        digitalWrite(Right1, LOW);
        digitalWrite(Left1, HIGH);
      }
      else                                                     //reverse
      {
        digitalWrite(Right1, HIGH);
        digitalWrite(Left1, LOW);
      }
      analogWrite(Right2,RightAmount);                          //Left motor forward
    } 
    
    */
}


